#!/bin/bash

#    Copyright (C) 2011 Gilles DOFFE
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

print_usage()
{
        echo "This script build xenomai packages for debian/ubuntu distros.

./build_xeno_kernel [-a <arch>] [-h] [-c <kernel_config_file>] [-d] [-k <kernel_version>]

Options:
        -a <arch>               Wanted architecture
        -h                      Display this help message
        -c <kernel_config_file> Use this kernel configuration file to apply
        -d                      Download development version of Xenomai
        -k <kernel_version>     Use this kernel version"
}

parse_options()
{
        while getopts “a:c:dhk:p:” option
        do
        case ${option} in
          h)
                print_usage
                exit 1
                ;;
          a)    target=${OPTARG}
                ;;
          c)
                config_file="$(cd $(dirname ${OPTARG}) && pwd)/$(basename ${OPTARG})"
                options="${options} -c $(basename ${OPTARG})"
                ;;
          d)
                dev_version=1
                options="${options} -d"
                ;;
          k)
                kernel_version=${OPTARG}
                options="${options} -k ${OPTARG}"
                ;;
          p)
                working_directory=${OPTARG}
                #options="${options} -p ${OPTARG}"
                ;;
          ?)
                print_usage
                exit
                ;;
        esac
        done
}


# Download last stable xenomai
get_xenomai()
{
        if [ "1" = "${dev_version}" ]; then
                git clone git://git.xenomai.org/xenomai-head.git
                xenomai_version="head"
        else
                echo "Downloading ..."
                wget -q -nd -c -r -np -nc -A "LATEST-IS-v*.tar.bz2" http://download.gna.org/xenomai/stable/
                # Untar xenomai
                xenomai_file=$(ls LATEST-IS-v*.tar.bz2 | sort | tail -1)
                xenomai_version=$(echo ${xenomai_file:11} | sed s/.tar.bz2//)
                tar xjpf LATEST-IS-v${xenomai_version}.tar.bz2
        fi

        xenomai_directory=xenomai-${xenomai_version}
}

# Download last stable adeos patch
get_adeos_patch()
{
        if [ "" != "${kernel_version}" ]; then
                wget -nv -nd -c -r -np -nc -A "adeos-ipipe-${kernel_version}-${arch}-*.patch" http://download.gna.org/adeos/patches/v2.6/${arch}/
                mv adeos-ipipe-*-${arch}-*.patch xenomai-${xenomai_version}/ksrc/arch/${arch}/patches/
                adeos_patch=$(ls -1 xenomai-${xenomai_version}/ksrc/arch/${arch}/patches/ | grep "adeos-ipipe-${kernel_version}-${arch}-.*\.patch" | sort | tail -1)
        else
                wget -nv -nd -c -r -np -nc -A "adeos-ipipe-*-${arch}-*.patch" http://download.gna.org/adeos/patches/v2.6/${arch}/
                mv adeos-ipipe-*-${arch}-*.patch xenomai-${xenomai_version}/ksrc/arch/${arch}/patches/
                adeos_patch=$(ls -1 xenomai-${xenomai_version}/ksrc/arch/${arch}/patches/ | grep "adeos-ipipe-.*-${arch}-.*\.patch" | sort | tail -1)
        
                # Extract kernel version from patch file name if not specified
                kernel_version=$(echo ${adeos_patch} | cut -d'-' -f3)
        fi
}

# Download the kernel
get_kernel()
{
        kernel_url="http://www.kernel.org/pub/linux/kernel/v2.6/linux-${kernel_version}.tar.bz2"
        wget -c -nc $kernel_url
}

# Build kernel
build_kernel_package()
{
        # Untar the kernel
        if [[ -d linux-${kernel_version} ]]; then
                answer=""
                while [ "y" != "${answer}" ] && [ "n" != "${answer}" ]; do
                        echo -n "linux-${kernel_version} directory already exists, delete it (y/n) ? "
                        read answer
                        answer=$(echo ${answer} | tr 'YN' 'yn')
                done
                if [ "y" = "${answer}" ]; then
                        rm -Rf linux-${kernel_version}
                else
                        echo "Exiting then..."
                        exit 0;
                fi
        fi
        tar -xjpf linux-${kernel_version}.tar.bz2
        
        # Patch the kernel
        (cd ${xenomai_directory} && scripts/prepare-kernel.sh --linux=../linux-${kernel_version}/ --adeos=ksrc/arch/${arch}/patches/${adeos_patch} --arch=${arch})
        
        # Configure kernel
        if [ "" != "${config_file}" ]; then
                cp -v ${config_file} linux-${kernel_version}/.config
        else
                (cd linux-${kernel_version} && make menuconfig)
        fi
        
        # Build
        (cd linux-${kernel_version} && fakeroot make-kpkg --initrd --uc --us kernel_image kernel_headers)
}


# Build xenomai
build_xenomai_package()
{
        (cd ${xenomai_directory} && debchange -v ${xenomai_version} Release ${xenomai_version})
        (cd ${xenomai_directory} && debuild -uc -us)
}

local_build()
{
        cd ${working_directory}

        get_xenomai
        get_adeos_patch
        get_kernel
        build_kernel_package
        build_xenomai_package
}

### MAIN ###
target=$(arch)
working_directory=${PWD}

parse_options ${@}

if [ "" != "$(echo ${target} | grep "i.86")" ]; then
        arch=x86
elif [ "" != "$(echo ${target} | grep "x86_64")" ]; then
        arch=x86
elif [ "" != "$(echo ${target} | grep "arm")" ]; then
        arch=arm
else
        echo "Unknown arch, exiting."
        exit 1
fi

if [ "$(arch)" != "${target}" ]; then
        if [[ -d /tmp/bootstrap ]]; then
                answer=""
                while [ "y" != "${answer}" ] && [ "n" != "${answer}" ]; do
                        echo -n "/tmp/bootstrap directory already exists, rebuild it ? (y/n) ? "
                        read answer
                        answer=$(echo ${answer} | tr 'YN' 'yn')
                done
                if [ "y" = "${answer}" ]; then
                        rm -Rf /tmp/bootstrap
                        qemu-debootstrap --arch ${target} --components="main,restricted,universe,multiverse" --include="build-essential,wget,libncurses5-dev,kernel-package,fakeroot,devscripts,debhelper,dh-kpatches" precise /tmp/bootstrap
                fi
        else
            qemu-debootstrap --arch ${target} --components="main,restricted,universe,multiverse" --include="build-essential,wget,libncurses5-dev,kernel-package,fakeroot,devscripts,debhelper,dh-kpatches" precise /tmp/bootstrap
        fi

        if [ "${?}" != "0" ]; then
                echo "Debootstrap failed, exiting." > /dev/stderr
                exit 1
        fi

        if [ "${config_file}" != "0" ]; then
            cp -f ${config_file} /tmp/bootstrap/
        fi
        script=${0}
        cp ${script} /tmp/bootstrap/${script}
        cat ~/.wgetrc >> /tmp/bootstrap/etc/wgetrc
        LC_ALL=C chroot  /tmp/bootstrap/ /${script} ${options} -p /tmp
        cp /tmp/bootstrap/tmp/*.deb .
else
        local_build
fi

